/*
 * PrintF.c
 *	Funciones de entrada/salida de texto por el puerto Serie.
 *  Created on: 21/3/2015
 *      Author: David Otal
 */
#include "stdio.h"
#include "stdarg.h"
#include "string.h"
#include "LPC17xx.h"
#include "type.h"
#include "FreeRTOS.h"
#include "uart.h"
#include "semphr.h"
#include "PrintF.h"

uint32_t numPort;
uint32_t velocitat;
xSemaphoreHandle xPrint0Semaphore;
xSemaphoreHandle xPrint1Semaphore;
xSemaphoreHandle xPrint3Semaphore;
xSemaphoreHandle xScanSemaphore;
xSemaphoreHandle xScan0Semaphore;
xSemaphoreHandle xScan1Semaphore;
xSemaphoreHandle xScan3Semaphore;

/** Realitza una espera activa
 * @param	ms		Milisegons aproximats d'espera
 * @return 	void
 */
void _delayMs(uint32_t ms) {
	int i;
	// Implementació de delayMS. Espera activa.
	// 7140 iterations are 1ms aprox (it's a vulgar implementation)
	for(i=0; i<( ((uint32_t)7140)*ms); i++){
		;
	}
};

/** Neteja el buffer d'entrada de dades de la UART
 * @param	port		Port de la UART (0, 1 o 3)
 * @return 	void
 */
void _clearBuffUart(uint8_t port){
	extern volatile uint32_t UART0Count;
	extern volatile uint32_t UART1Count;
	extern volatile uint32_t UART3Count;
	int i;
	char * apBuffer;
	if (port == 0) {
		UART0Count = 0;
		apBuffer = (char *) &UART0Buffer;
	}
	else if (port == 1) {
		UART1Count = 0;
		apBuffer = (char *) &UART1Buffer;
	}
	else if (port == 3) {
		UART3Count = 0;
		apBuffer = (char *) &UART3Buffer;
	}
	for(i=0; i < BUFSIZE; i++){
		*apBuffer = 0;
		apBuffer++;
	}
}

/** Habilita les interrupcions dels semafors
 * @return 	void
 */
static void prvSetupSoftwareInterrupt()
{
	NVIC_SetPriority( mainSW_INTERRUPT_ID, mainSOFTWARE_INTERRUPT_PRIORITY );
	NVIC_EnableIRQ( mainSW_INTERRUPT_ID ); /* Habilita Interrupció */
}

/** Gestiona les interrupcions dels semafors
 * @return 	void
 */
void vSoftwareInterruptHandler( void )
{
	portBASE_TYPE xHigherPriorityTaskWoken = pdFALSE;
    /* 'Give' the semaphore to unblock the task. */
    xSemaphoreGiveFromISR( xPrint0Semaphore, &xHigherPriorityTaskWoken );
    xSemaphoreGiveFromISR( xPrint1Semaphore, &xHigherPriorityTaskWoken );
    xSemaphoreGiveFromISR( xPrint3Semaphore, &xHigherPriorityTaskWoken );
    xSemaphoreGiveFromISR( xScanSemaphore, &xHigherPriorityTaskWoken );
    /* Clear the software interrupt bit using the interrupt controllers
    Clear Pending register. */
    mainCLEAR_INTERRUPT();

    /* Giving the semaphore may have unblocked a task - if it did and the
    unblocked task has a priority equal to or above the currently executing
    task then xHigherPriorityTaskWoken will have been set to pdTRUE and
    portEND_SWITCHING_ISR() will force a context switch to the newly unblocked
    higher priority task.

    NOTE: The syntax for forcing a context switch within an ISR varies between
    FreeRTOS ports.  The portEND_SWITCHING_ISR() macro is provided as part of
    the Cortex-M3 port layer for this purpose.  taskYIELD() must never be called
    from an ISR! */
    portEND_SWITCHING_ISR( xHigherPriorityTaskWoken );
}

void PrintF_start(uint32_t portNum, uint32_t baudrate){
    /* Es crean els semafors */
    vSemaphoreCreateBinary( xPrint0Semaphore );
    vSemaphoreCreateBinary( xPrint1Semaphore );
    vSemaphoreCreateBinary( xPrint3Semaphore );
    vSemaphoreCreateBinary( xScanSemaphore );
    vSemaphoreCreateBinary( xScan0Semaphore );

    /* Habilita interrupcions del semafor. */
    prvSetupSoftwareInterrupt();
	numPort = portNum;
	velocitat= baudrate;
	UARTInit(portNum, baudrate);
}

void PrintF_print(uint8_t port, const char * format, ... ){
	va_list lv; 		// Llista de paràmetres
	xSemaphoreHandle xPrintSemaphore;
	char edit_text[255];
	char * ptext = edit_text;
	if ( port==0 ){ xPrintSemaphore = xPrint0Semaphore;	}
	else if ( port==1 ){ xPrintSemaphore = xPrint1Semaphore; }
	else if ( port==3 ){ xPrintSemaphore = xPrint3Semaphore; }
    if( xPrintSemaphore != NULL ) {
    	while ( xSemaphoreTake( xPrintSemaphore, 10 ) == pdFALSE){
    		;
    	}
		va_start( lv, format);
		vsnprintf(ptext, 255, format,  lv );
		UARTSend( port, (uint8_t *)edit_text, strlen(edit_text) );
		va_end( lv );
		xSemaphoreGive( xPrintSemaphore );
    }
}

void PrintF_stream(uint8_t port, char * stream, int size ){
	xSemaphoreHandle xPrintSemaphore;
	if ( port==0 ){ xPrintSemaphore = xPrint0Semaphore;	}
	else if ( port==1 ){ xPrintSemaphore = xPrint1Semaphore; }
	else if ( port==3 ){ xPrintSemaphore = xPrint3Semaphore; }
    if( xPrintSemaphore != NULL ) {
    	while ( xSemaphoreTake( xPrintSemaphore, 10 ) == pdFALSE){
    		;
    	}
		UARTSend( port, (uint8_t *)stream, size );
		xSemaphoreGive( xPrintSemaphore );
    }
}

int PrintF_scan(uint8_t port, char * text, uint32_t timeout){
	extern volatile uint32_t UART0Count;
	extern volatile uint32_t UART1Count;
	extern volatile uint32_t UART3Count;
	int oldCount=-1;
	int newCount=0;
	int result = 0;
	int i = 0;
	char * apBuffer;
	if( xScanSemaphore != NULL ) {
      	if ( xSemaphoreTake( xScanSemaphore, 0 ) == pdTRUE){
      		if ( port == 0 ){
      			newCount=UART0Count;
      			apBuffer = (char *) &UART0Buffer;
      			UART0Count = 0; 	// clean buff
      			LPC_UART0->IER = IER_THRE | IER_RLS | IER_RBR; 	// enable interrupt RX UART3
      			_delayMs(timeout); 	// wait sometime for receive some commands
      			do{
      				oldCount = newCount;
      				_delayMs(timeout);
      				newCount = UART0Count;
      			}while(oldCount != newCount); // wait a little time while we have new chars
      			if ( UART0Count > 0) {
      				result = UART0Count;
      				for (i=0; i < result; i++ )
      				{ text[i] = apBuffer[i]; }
      			} else {
      				text = "/0";
      				newCount = 0;
      			}
      		}
      		else if ( port == 1 ){
      			size_t newCount=UART1Count;
      			apBuffer = (char *) &UART1Buffer;
      			UART1Count = 0; 	// clean buff
      			LPC_UART1->IER = IER_THRE | IER_RLS | IER_RBR; 	// enable interrupt RX UART3
      			_delayMs(timeout); 	// wait sometime for receive some commands
      			do{
      				oldCount = newCount;
      				_delayMs(timeout);
      				newCount = UART1Count;
      			}while(oldCount != newCount); // wait a little time while we have new chars
      			if ( UART1Count > 0) {
      				result = UART1Count;
      				for (i=0; i < result; i++ )
      				{ text[i] = apBuffer[i]; }
      			} else {
      				text = "/0";
      				newCount = 0;
      			}
      			LPC_UART1->IER = IER_THRE | IER_RLS; 	// disable interrupt RX UART0
      		}
      		else if ( port == 3 ){
      			size_t newCount=UART3Count;
      			apBuffer = (char *) &UART3Buffer;
      			UART3Count = 0; 	// clean buff
      			LPC_UART3->IER = IER_THRE | IER_RLS | IER_RBR; 	// enable interrupt RX UART3
      			_delayMs(timeout); 	// wait sometime for receive some commands
      			do{
      				oldCount = newCount;
      				_delayMs(timeout);
      				newCount = UART3Count;
      			}while(oldCount != newCount); // wait a little time while we have new chars
      			if ( UART3Count > 0) {
      				//strncpy(text, apBuffer, UART3Count );
      				result = UART3Count;
      				for (i=0; i < result; i++ )
      				{ text[i] = apBuffer[i]; }
      			} else {
      				text = "/0";
      				result = 0;
      			}
      			LPC_UART3->IER = IER_THRE | IER_RLS; 	// disable interrupt RX UART0
      		}
      		_clearBuffUart(port);
      		xSemaphoreGive( xScanSemaphore );
      	}
	}
	return (result);
}
